<?php

/**
 * BIPKampany HTTP API-t megvalosito osztaly SMS kuldeshez
 * (c) Bip Communications Kft.
 * Minden jog fenntartva
 *
 * @package BipKampany
 * @subpackage API
 * @copyright Bip Communications Kft. 
 * @link http://www.bipkampany.hu
 */

class BipKampanyAPI {

  /**
   * API meghivasahoz hasznalt modszer. Lehetseges ertekei: 'file_get_contents' vagy 'curl'
   * 
   * @access public
   * @var string
   */
  var $callingMethod = 'file_get_contents';

  /**
   * Naplofile neve. Csak akkor vegez naplozast az API, ha 
   * kitoltjuk a filenevet.
   *
   * @var string
   */
  var $logFilename = false;

  /**
   * Az API meghivasat kovetoen ebben a valtozoban taroljuk a legutolso
   * API valaszt (hibauzenet kodjat, szoveget).
   * 
   * @access public
   * @var string
   */ 
  var $lastResult;

  /**
   * API URL
   *
   * @access private
   * @var string
   */
  var $url = 'http://api.bipkampany.hu/';

  /**
   * key => value parameterparok, amelyeket az API-nak adunk at.
   *
   * @access private
   * @var string
   */
  var $params   = array();

  /**
   * BipKampanyAPI konstruktor.
   *
   * @access public
   * @param string $key      hozzafereshez kapott API kulcs
   * @return void
   */
  function __construct( $key ) {

    $this->params['key'] = $key;

  }

  /**
   * SMS feladasa. Hiba eseten exception-t valt ki.
   *
   * @access public
   * @param string $number      Mobiltelefonszam nemzetkozi formatumban (pl. 36201234567)
   * @param string $message     Uzenet UTF-8 kodolassal
   * @param string $senderid    Feladoazonosito (csak a webes feluleten elfogadtatott azonositok hasznalhatoak)
   * @param string $timetosend  Idozites idopontja, opcionalis.
   * @param string $type        Uzenet tipusa, lehetseges erteke 'unicode'.
   * @param string $referenceid A callback hivas soran a BIP altal valtozatlan formaban visszaadott tetszoleges azonosito
   * @param string $callback    A callback hivas soran visszaadando allapotok felsorolasa, vagy ALL minden statusz keresehez.
   * @return void
   */
  function sendSMS( $number, $message, $senderid, $timetosend = null, $type = null, $callback = null, $referenceid = null ) {

    $params             = $this->params;
    $params['message']  = $message;
    $params['number']   = $number;
    $params['senderid'] = $senderid;

    if ( $timetosend !== null )
      $params['timetosend'] = $timetosend;

    if ( $type !== null )
      $params['type'] = $type;

    if ( $referenceid !== null ) {
      if ( !strlen( $callback ) )
        throw new Exception('BipKampanyAPI: $referenceid is used without specifying the expected callback status(es) in $callback');
      $params['referenceid'] = $referenceid;
    }

    if ( $callback !== null )
      $params['callback'] = $callback;

    $result = $this->call( 'sendsms', $params );
    return $this->parseResult( $result );

  }

  /**
   * Egyenleg lekerdezese
   *
   * @access public
   * @return array A valasz PHP tomb formaban, balance es currency tombindexeiben az egyenleggel
   */
  function getBalance() {

    $result = $this->call( 'getbalance', $this->params );
    return $this->parseResult( $result );

  }

  /**
   * Tamogatott karakterek lekerdezese
   *
   * @access public
   * @return string A tamogatott karakterek egyetlen sztringbe fuzve, UTF-8 kodolasban
   */
  function getCharset() {

    $result = $this->call( 'getcharset', $this->params );
    return $this->parseResult( $result );

  }

  /**
   * Bejovo uzenetek lekerdezese
   *
   * @access public
   * @return array A valasz PHP tomb formaban, messages tombindexben a bejovo uzenetekkel (csokkeno idorendben)
   */
  function getIncomingSMSes( $limit = 10 ) {

    $params = $this->params;

    if ( $limit !== null )
      $params['limit'] = $limit;

    $result = $this->call( 'getincomingsmses', $params );
    return $this->parseResult( $result );

  }

  /**
   * Idozitett uzenet visszavonasa
   *
   * @access public
   * @param string $referenceid Az a referenciaid, amit az uzenet feladaskor megadtunk. Tobb referenciaid is megadhato vesszovel elvalasztva, a sztring byteban merve nem lehet 65000 bytenal nagyobb.
   * @return array A valasz PHP tomb formaban.
   */
  function cancelSMS( $referenceids ) {

    $params = $this->params;
    $params['referenceid'] = $referenceids;

    $result = $this->call( 'cancelsms', $params );
    return $this->parseResult( $result );

  }

  /**
   * Az utolso API valasz tartalma tombkent vagy null, ha meg nem volt valasz
   * (a tombindexek: 'result' (erteke 'OK' vagy 'ERR'), 'code' es 'message')
   *
   * @access public
   * @return mixed Az utolso API valasz tartalma tombkent, vagy null
   */
  function getLastResult() {

    return $this->lastResult;

  }

  /**
   * API hivas vegrehajtasa
   *
   * @access private
   * @param string $params Az API-nak atadando parameterek
   * @return string A kapott valasz
   */
  function call( $action, $params ) {

    // egyes szervereken az arg_separator.output beallitas erteke
    // "&amp;", ezt elkerulendo vegigparameterezzuk a http_build_query-t
    $url = $this->url . $action . '?' . http_build_query( $params, '', '&' );

    $this->log( $this->callingMethod . ': ' . $url );

    switch ( $this->callingMethod ) {
      case 'file_get_contents':
        if ( !ini_get('allow_url_fopen') )
          throw new Exception('BipKampanyAPI: - allow_url_fopen must be On to use "file_get_contents" callingMethod. You may use curl callingMethod if you have CURL extension installed.');
        $result = file_get_contents( $url );
        break;
      case 'curl':
        if ( !extension_loaded('curl') )
          throw new Exception('BipKampanyAPI: CURL extension must be installed to use "curl" callingMethod.');
        $result = $this->callCURL( $url );
        break;
      default:
        throw new Exception('BipKampanyAPI: unimplemented callingMethod: "' . $this->callingMethod . '"');
        break;
    }

    if ( $result === false )
      throw new Exception( 'BipKampanyAPI: file_get_contents("' . $url . '") failed' );

    return $result;
      
  }

  /**
   * API hivas vegrehajtasa CURL segitsegevel
   *
   * @access private
   * @param string $params Az API-nak atadando parameterek
   * @return string A kapott valasz
   */
  function callCURL( $url ) {

    $curl = curl_init();

    curl_setopt_array( $curl, array(
      CURLOPT_URL               => $url,
      // CURLOPT_CONNECTTIMEOUT_MS => 2000,
      // CURLOPT_TIMEOUT           => 10,
      CURLOPT_FAILONERROR       => true,
      CURLOPT_RETURNTRANSFER    => true
    ) );

    $html     = curl_exec( $curl );
    $httpcode = curl_getinfo( $curl, CURLINFO_HTTP_CODE );

    if ( $html === false ) {

      throw new Exception(
        'BipKampanyAPI: CURL error, HTTP error code: ' . $httpcode . ', ' . curl_error( $curl )
      );

    } 
    else
      return $html;

  }
  
  /**
   * API altal visszaadott valasz feldolgozasa
   *
   * @access private
   * @param string $result Az API altal visszaadott valasz
   * @return array A valasz tomb formaban
   */
  function parseResult( $result ) {

    parse_str( $result, $resultparts );
    $this->lastResult = $resultparts;
    $this->log( var_export( $resultparts, true ) );

    switch ( @$resultparts['result'] ) {

      case 'OK':
        // SMS submitted successfully
        return $resultparts;
        break;

      case 'ERR':
        // error during SMS submit
        throw new BipKampanyAPIException(
          $resultparts['message'],
          $resultparts['code']
        );
        break;

      default:
        throw new Exception(
          'BipKampanyAPI: unimplemented result '.
          '"' . @$resultparts['code'] . '", ' .
          '"' . @$resultparts['message'] . '", '.
          'raw result: "' . $result . '"'
        );
        break;

    }

  }

  /**
   * Bejegyzes irasai naploallomanyba, ha szukseges.
   * Ha nem definialtak a $this->filename attributumot, akkor
   * nem vegez logolast a metodus.
   *
   * @access private
   * @param string $result A naplozando sztring
   * @return void
   */
  function log( $string ) {

    if ( $this->logFilename ) {

      $f = fopen( $this->logFilename, 'a' );

      if ( !$f )
        throw new Exception('BipKampanyAPI: fopen( "' . $this->logFilename . '" ) failed' );

      fputs( $f, date("Y-m-d H:i:s") . ' - ' . $string . "\n" );
      fclose( $f );

    }

  }

}

/**
 * BIP API Exception
 */
class BipKampanyAPIException extends Exception {

  public function __construct( $message, $code ) {
    $this->message = $message;
    $this->code    = $code;
    parent::__construct( $message, $code );
  }

  public function __toString() {
    return __CLASS__ . ": BIP_API_ERROR #{$this->code}, {$this->message}\n";
  }

}

?>